import 'package:async_injector/error/dependency_is_null.dart';
import 'package:async_injector/module/module.dart';
import 'package:async_injector/module/module_impl.dart';
import 'package:async_injector/provide/provider.dart';
import 'package:async_injector/provide/scope.dart';

/// [Module] builder
/// where they can request the dependencies they need to init
class ProviderBuilder {
  final _scope = Scope();
  final _initQueue = <Module>[];

  /// adding a module to the init queue
  module(Module module) {
    _initQueue.add(module);
  }

  /// create [ModuleImpl] and add to queue
  moduleImpl<T>(dynamic Function(Provider) onInit) {
    module(ModuleImpl<T>(onInit));
  }

  /// adding a dependency to the scope
  dependency<T>(T value) {
    _scope.add<T>(value);
  }

  /// adding dependencies that will not be used after assembly
  buildDependency<T>(T value) {
    _scope.addBuildDependencies<T>(value);
  }

  /// init of [Module]s
  Future<Provider> build() async {
    await _initDependencies();
    return _scope;
  }

  _initDependencies() async {
    for (Module module in _initQueue) {
      var moduleInit = module.init(_scope);

      var dependency =
          moduleInit is Future<dynamic> ? await moduleInit : moduleInit;

      if (dependency == null) {
        throw DependencyIsNull(module);
      }
      _scope.add(dependency, type: module.type);
    }
    _scope.clearAfterBuild();
    _initQueue.clear();
  }
}
