import 'package:async_injector/error/not_have_dependency.dart';
import 'package:async_injector/provide/provider.dart';

/// dependency store
/// can cast [Provider] to this
class Scope extends Provider {
  final Map<String, dynamic> _dependencies = {};

  /// clearing after build
  final Map<String, dynamic> _buildDependencies = {};

  @override
  T get<T>() {
    T dependency;

    if (_buildDependencies != null) {
      dependency = _buildDependencies[T.toString()];
    }
    dependency ??= _dependencies[T.toString()];

    if (dependency == null) {
      throw NotHaveDependency("not have ${T.toString()}");
    }

    return dependency;
  }

  @override
  add<T>(T dependency, {Type type}) {
    var key = type != null && type.toString() != dynamic.toString()
        ? type.toString()
        : createKey<T>(dependency);
    _dependencies[key] = dependency;
  }

  addBuildDependencies<T>(T dependency) {
    var key = createKey<T>(dependency);
    _buildDependencies[key] = dependency;
  }

  clearAfterBuild() {
    _buildDependencies.clear();
  }

  createKey<T>(T dependency) => T.toString() != dynamic.toString()
      ? T.toString()
      : dependency.runtimeType.toString();
}
